@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/lookups/dbmdb.c,v 1.3 2005/06/27 14:29:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -56,17 +58,17 @@ rc = lf_check_file(-1, filename, S_IFREG, modemask, owners, owngroups,
 #else
   {
   uschar filebuffer[256];
-  sprintf(CS filebuffer, "%.250s.db", filename);
+  (void)sprintf(CS filebuffer, "%.250s.db", filename);
   rc = lf_check_file(-1, filebuffer, S_IFREG, modemask, owners, owngroups,
     "dbm", errmsg);
   if (rc < 0)        /* stat() failed */
     {
-    sprintf(CS filebuffer, "%.250s.dir", filename);
+    (void)sprintf(CS filebuffer, "%.250s.dir", filename);
     rc = lf_check_file(-1, filebuffer, S_IFREG, modemask, owners, owngroups,
       "dbm", errmsg);
     if (rc == 0)     /* x.dir was OK */
       {
-      sprintf(CS filebuffer, "%.250s.pag", filename);
+      (void)sprintf(CS filebuffer, "%.250s.pag", filename);
       rc = lf_check_file(-1, filebuffer, S_IFREG, modemask, owners, owngroups,
         "dbm", errmsg);
       }
@@ -88,13 +90,14 @@ the keylength in order to include the terminating zero. */
 
 int
 dbmdb_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
 EXIM_DB *d = (EXIM_DB *)handle;
 EXIM_DATUM key, data;
 
 filename = filename;    /* Keep picky compilers happy */
 errmsg = errmsg;
+do_cache = do_cache;
 
 EXIM_DATUM_INIT(key);               /* Some DBM libraries require datums to */
 EXIM_DATUM_INIT(data);              /* be cleared before use. */
@@ -120,9 +123,10 @@ return FAIL;
 
 int
 dbmnz_find(void *handle, uschar *filename, uschar *keystring, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
-return dbmdb_find(handle, filename, keystring, length-1, result, errmsg);
+return dbmdb_find(handle, filename, keystring, length-1, result, errmsg,
+  do_cache);
 }
 
 
