@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/match.c,v 1.7 2005/06/27 14:29:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for matching strings */
@@ -60,6 +62,9 @@ The possible types of pattern are:
       @mx_secondary  matches any domain with a secondary MX to the local host
   . literal - anything else
 
+Any of the @mx_xxx options can be followed by "/ignore=<list>" where <list> is
+a list of IP addresses that are to be ignored (typically 127.0.0.1).
+
 Arguments:
   arg            check_string_block pointer - see below
   pattern        the pattern to be matched
@@ -157,12 +162,13 @@ just fall through - the match will fail. */
 
 if (cb->at_is_special && pattern[0] == '@')
   {
-  int prim = -1;
-  int secy = -1;
+  if (pattern[1] == 0)
+    {
+    pattern = primary_hostname;
+    goto NOT_AT_SPECIAL;               /* Handle as exact string match */
+    }
 
-  if (pattern[1] == 0) pattern = primary_hostname;  /* Fall through to exact */
-
-  else if (Ustrcmp(pattern, "@[]") == 0)
+  if (Ustrcmp(pattern, "@[]") == 0)
     {
     ip_address_item *ip;
     int slen = Ustrlen(s);
@@ -172,26 +178,44 @@ if (cb->at_is_special && pattern[0] == '@')
     return FAIL;
     }
 
-  else if (strcmpic(pattern, US"@mx_any") == 0 ||
-            (prim = strcmpic(pattern, US"@mx_primary")) == 0 ||
-            (secy = strcmpic(pattern, US"@mx_secondary")) == 0)
+  if (strncmpic(pattern, US"@mx_", 4) == 0)
     {
-    host_item h;
-    BOOL removed = FALSE;
     int rc;
+    host_item h;
+    BOOL prim = FALSE;
+    BOOL secy = FALSE;
+    BOOL removed = FALSE;
+    uschar *ss = pattern + 4;
+    uschar *ignore_target_hosts = NULL;
+
+    if (strncmpic(ss, US"any", 3) == 0) ss += 3;
+    else if (strncmpic(ss, US"primary", 7) == 0)
+      {
+      ss += 7;
+      prim = TRUE;
+      }
+    else if (strncmpic(ss, US"secondary", 9) == 0)
+      {
+      ss += 9;
+      secy = TRUE;
+      }
+    else goto NOT_AT_SPECIAL;
+
+    if (strncmpic(ss, US"/ignore=", 8) == 0) ignore_target_hosts = ss + 8;
+      else if (*ss != 0) goto NOT_AT_SPECIAL;
 
     h.next = NULL;
     h.name = s;
     h.address = NULL;
 
     rc = host_find_bydns(&h,
-      NULL,       /* ignore list */
-      TRUE,       /* MX required */
-      FALSE,      /* not A only */
-      FALSE,      /* no widening */
-      FALSE,      /* no widening */
-      NULL,       /* no feedback FQDN */
-      &removed);  /* feedback if local removed */
+      ignore_target_hosts,
+      HOST_FIND_BY_MX,     /* search only for MX, not SRV or A */
+      NULL,                /* service name not relevant */
+      NULL,                /* srv_fail_domains not relevant */
+      NULL,                /* mx_fail_domains not relevant */
+      NULL,                /* no feedback FQDN */
+      &removed);           /* feedback if local removed */
 
     if (rc == HOST_FIND_AGAIN)
       {
@@ -199,8 +223,8 @@ if (cb->at_is_special && pattern[0] == '@')
       return DEFER;
       }
 
-    if (rc == HOST_FOUND_LOCAL && secy != 0 ) return OK;
-    if (prim == 0) return FAIL;
+    if (rc == HOST_FOUND_LOCAL && !secy) return OK;
+    if (prim) return FAIL;
     return removed? OK : FAIL;
 
     /*** The above line used to be the following line, but this is incorrect,
@@ -213,6 +237,10 @@ if (cb->at_is_special && pattern[0] == '@')
     }
   }
 
+/* Escape point from code for specials that start with "@" */
+
+NOT_AT_SPECIAL:
+
 /* This is an exact string match if there is no semicolon in the pattern. */
 
 if ((semicolon = Ustrchr(pattern, ';')) == NULL)
@@ -307,6 +335,41 @@ return check_string(&cb, pattern, valueptr, NULL);
 
 
 
+/*************************************************
+*       Get key string from check block          *
+*************************************************/
+
+/* When caching the data from a lookup for a named list, we have to save the
+key that was found, because other lookups of different keys on the same list
+may occur. This function has knowledge of the different lookup types, and
+extracts the appropriate key.
+
+Arguments:
+  arg          the check block
+  type         MCL_STRING, MCL_DOMAIN, MCL_HOST, MCL_ADDRESS, or MCL_LOCALPART
+*/
+
+static uschar *
+get_check_key(void *arg, int type)
+{
+switch(type)
+  {
+  case MCL_STRING:
+  case MCL_DOMAIN:
+  case MCL_LOCALPART:
+  return ((check_string_block *)arg)->subject;
+
+  case MCL_HOST:
+  return ((check_host_block *)arg)->host_address;
+
+  case MCL_ADDRESS:
+  return ((check_address_block *)arg)->address;
+  }
+return US"";  /* In practice, should never happen */
+}
+
+
+
 /*************************************************
 *       Scan list and run matching function      *
 *************************************************/
@@ -332,9 +395,7 @@ Arguments:
   listptr      pointer to the pointer to the list
   sep          separator character for string_nextinlist();
                  normally zero for a standard list;
-                 sometimes UCHAR_MAX+n for single items;
-                 UCHAR_MAX+1 is used when the item is to be expanded
-                 UCHAR_MAX+2 is used when the item is not to be expanded
+                 sometimes UCHAR_MAX+1 for single items;
   anchorptr    -> tree of named items, or NULL if no named items
   cache_ptr    pointer to pointer to cache bits for named items, or
                  pointer to NULL if not caching; may get set NULL if an
@@ -344,6 +405,8 @@ Arguments:
                  in the structure it points to
   type         MCL_STRING, MCL_DOMAIN, MCL_HOST, MCL_ADDRESS, or MCL_LOCALPART
                  these are used for some special handling
+               MCL_NOEXPAND (whose value is greater than any of them) may
+                 be added to any value to suppress expansion of the list
   name         string to use in debugging info
   valueptr     where to pass back data from a lookup
 
@@ -363,6 +426,7 @@ match_check_list(uschar **listptr, int sep, tree_node **anchorptr,
 int yield = OK;
 unsigned int *original_cache_bits = *cache_ptr;
 BOOL include_unknown = FALSE;
+BOOL ignore_unknown = FALSE;
 uschar *list;
 uschar *sss;
 uschar *ot = NULL;
@@ -376,23 +440,27 @@ HDEBUG(D_any)
   if (listname[0] != 0) ot = string_sprintf("%s in %s?", name, listname);
   }
 
-/* If the list is empty, the answer is no. This case happens only when checking
-named options, so ot will be set. */
+/* If the list is empty, the answer is no. Skip the debugging output for
+an unnamed list. */
 
 if (*listptr == NULL)
   {
-  HDEBUG(D_lists) debug_printf("%s no (option unset)\n", ot);
+  HDEBUG(D_lists)
+    {
+    if (ot != NULL) debug_printf("%s no (option unset)\n", ot);
+    }
   return FAIL;
   }
 
 /* Expand the list before we scan it. A forced expansion gives the answer
-"not in list"; other expansion errors cause DEFER to be returned. If the
-separator is set to UCHAR_MAX+2 we have a single item list that is not to
-be expanded. */
+"not in list"; other expansion errors cause DEFER to be returned. However,
+if the type value is greater than or equal to than MCL_NOEXPAND, do not expand
+the list. */
 
-if (sep == UCHAR_MAX+2)
+if (type >= MCL_NOEXPAND)
   {
   list = *listptr;
+  type -= MCL_NOEXPAND;       /* Remove the "no expand" flag */
   }
 else
   {
@@ -464,6 +532,13 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
     if (Ustrcmp(ss, "+include_unknown") == 0)
       {
       include_unknown = TRUE;
+      ignore_unknown = FALSE;
+      continue;
+      }
+    if (Ustrcmp(ss, "+ignore_unknown") == 0)
+      {
+      ignore_unknown = TRUE;
+      include_unknown = FALSE;
       continue;
       }
     }
@@ -523,7 +598,6 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
 
       if (bits == 0)
         {
-        nb->cache_data = NULL;
         switch (match_check_list(&(nb->string), 0, anchorptr, &use_cache_bits,
                 func, arg, type, name, valueptr))
           {
@@ -533,7 +607,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
           }
 
         /* If this list was uncacheable, or a sublist turned out to be
-        uncacheable, the value of use_cache_lists will now be NULL, even if it
+        uncacheable, the value of use_cache_bits will now be NULL, even if it
         wasn't before. Ensure that this is passed up to the next level.
         Otherwise, remember the result of the search in the cache. */
 
@@ -544,9 +618,25 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
         else
           {
           use_cache_bits[offset] |= bits << shift;
+
           if (valueptr != NULL)
             {
-            nb->cache_data = *valueptr;
+            int old_pool = store_pool;
+            namedlist_cacheblock *p;
+
+            /* Cached data for hosts persists over more than one message,
+            so we use the permanent store pool */
+
+            store_pool = POOL_PERM;
+            p = store_get(sizeof(namedlist_cacheblock));
+            p->key = string_copy(get_check_key(arg, type));
+
+
+            p->data = (*valueptr == NULL)? NULL : string_copy(*valueptr);
+            store_pool = old_pool;
+
+            p->next = nb->cache_data;
+            nb->cache_data = p;
             if (*valueptr != NULL)
               {
               DEBUG(D_lists) debug_printf("data from lookup saved for "
@@ -556,7 +646,9 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
           }
         }
 
-       /* Previously cached */
+       /* Previously cached; to find a lookup value, search a chain of values
+       and compare keys. Typically, there is only one such, but it is possible
+       for different keys to have matched the same named list. */
 
       else
         {
@@ -565,7 +657,16 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
         cached = US" - cached";
         if (valueptr != NULL)
           {
-          *valueptr = nb->cache_data;
+          uschar *key = get_check_key(arg, type);
+          namedlist_cacheblock *p;
+          for (p = nb->cache_data; p != NULL; p = p->next)
+            {
+            if (Ustrcmp(key, p->key) == 0)
+              {
+              *valueptr = p->data;
+              break;
+              }
+            }
           DEBUG(D_lists) debug_printf("cached lookup data = %s\n", *valueptr);
           }
         }
@@ -596,16 +697,25 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
         case DEFER:
         goto DEFER_RETURN;
 
-        /* The ERROR return occurs only when checking hosts, when either a
-        forward or reverse lookup has failed. The error string gives details of
+        /* The ERROR return occurs when checking hosts, when either a forward
+        or reverse lookup has failed. It can also occur in a match_ip list if a
+        non-IP address item is encountered. The error string gives details of
         which it was. */
 
         case ERROR:
-        HDEBUG(D_lists)
-          debug_printf("%s %s (%s)\n", ot, include_unknown? "yes":"no", error);
-        if (!include_unknown) return FAIL;
-        log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
-        return OK;
+        if (ignore_unknown)
+          {
+          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown",
+            error);
+          }
+        else
+          {
+          HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
+            include_unknown? "yes":"no", error);
+          if (!include_unknown) return FAIL;
+          log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
+          return OK;
+          }
         }
       }
     }
@@ -673,22 +783,30 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
       switch ((func)(arg, ss, valueptr, &error))
         {
         case OK:
-        fclose(f);
+        (void)fclose(f);
         HDEBUG(D_lists) debug_printf("%s %s (matched \"%s\" in %s)\n", ot,
           (yield == OK)? "yes" : "no", sss, filename);
         return file_yield;
 
         case DEFER:
-        fclose(f);
+        (void)fclose(f);
         goto DEFER_RETURN;
 
         case ERROR:          /* host name lookup failed - this can only */
-        HDEBUG(D_lists)      /* be for an incoming host (not outgoing) */
-          debug_printf("%s %s (%s)\n", ot, include_unknown? "yes":"no", error);
-        fclose(f);
-        if (!include_unknown) return FAIL;
-        log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
-        return OK;
+        if (ignore_unknown)  /* be for an incoming host (not outgoing) */
+          {
+          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown",
+            error);
+          }
+        else
+         {
+          HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
+            include_unknown? "yes":"no", error);
+          (void)fclose(f);
+          if (!include_unknown) return FAIL;
+          log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
+          return OK;
+          }
         }
       }
 
@@ -696,7 +814,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
     for the file, in case this is the last item in the list. */
 
     yield = file_yield;
-    fclose(f);
+    (void)fclose(f);
     }
   }    /* Loop for the next item on the top-level list */
 
@@ -714,17 +832,15 @@ return DEFER;
 }
 
 
-
-
 /*************************************************
-*            Match in colon-separated list       *
+*          Match in colon-separated list         *
 *************************************************/
 
-/* This function is used for domain lists. It is not used for host lists or
-address lists, which have additional interpretation of the patterns. Some calls
-of it set sep > UCHAR_MAX in order to use its matching facilities on single
-items. When this is done, it arranges to set the numerical variables as a
-result of the match.
+/* This function is used for domain lists and local part lists. It is not used
+for host lists or address lists, which have additional interpretation of the
+patterns. Some calls of it set sep > UCHAR_MAX in order to use its matching
+facilities on single items. When this is done, it arranges to set the numerical
+variables as a result of the match.
 
 This function is now just a short interface to match_check_list(), which does
 list scanning in a general way. A good compiler will optimize the tail
@@ -740,6 +856,8 @@ Arguments:
                  MCL_LOCALPART when matching a local part list (address lists
                    have their own function)
                  MCL_STRING for others (e.g. list of ciphers)
+                 MCL_NOEXPAND (whose value is greater than any of them) may
+                   be added to any value to suppress expansion of the list
   caseless       TRUE for (mostly) caseless matching - passed directly to
                    match_check_string()
   valueptr       pointer to where any lookup data is to be passed back,
@@ -764,7 +882,7 @@ cb.subject = caseless? string_copylc(s) : string_copy(s);
 cb.expand_setup = (sep > UCHAR_MAX)? 0 : -1;
 cb.use_partial = TRUE;
 cb.caseless = caseless;
-cb.at_is_special = (type == MCL_DOMAIN);
+cb.at_is_special = (type == MCL_DOMAIN || type == MCL_DOMAIN + MCL_NOEXPAND);
 if (valueptr != NULL) *valueptr = NULL;
 return  match_check_list(listptr, sep, anchorptr, &local_cache_bits,
   check_string, &cb, type, s, valueptr);
@@ -783,7 +901,7 @@ its arguments are in an indirect block.
 Arguments:
   arg            the argument block (see below)
   pattern        the pattern to match
-  valueptr       where to return a value (never used)
+  valueptr       where to return a value
   error          for error messages (not used in this function; it never
                    returns ERROR)
 
@@ -813,37 +931,28 @@ uschar *subject = cb->address;
 uschar *s, *pdomain, *sdomain;
 
 error = error;  /* Keep clever compilers from complaining */
-valueptr = valueptr;
 
 DEBUG(D_lists) debug_printf("address match: subject=%s pattern=%s\n",
   subject, pattern);
 
-/* Handle a regular expression, which must match the entire incoming address. */
+/* Handle a regular expression, which must match the entire incoming address.
+This may be the empty address. */
 
 if (*pattern == '^')
   return match_check_string(subject, pattern, cb->expand_setup, TRUE,
     cb->caseless, FALSE, NULL);
 
-/* If the subject is the empty string, the only pattern it can match (other
-than a regular expression) is the empty pattern. */
-
-if (*subject == 0) return (*pattern == 0)? OK : FAIL;
-
-/* Find the domain in the subject */
-
-sdomain = Ustrrchr(subject, '@');
-
-/* Handle the case of a pattern that is just a lookup. Skip over possible
-lookup names (letters, digits, hyphens). Skip over a possible * or *@ at
-the end. Then we must have a semicolon for it to be a lookup. */
+/* Handle a pattern that is just a lookup. Skip over possible lookup names
+(letters, digits, hyphens). Skip over a possible * or *@ at the end. Then we
+must have a semicolon for it to be a lookup. */
 
 for (s = pattern; isalnum(*s) || *s == '-'; s++);
 if (*s == '*') s++;
 if (*s == '@') s++;
 
-/* If it is a straight lookup, do a lookup for the whole address. Partial
-matching doesn't make sense here, so we ignore it, but write a panic log entry.
-However, *@ matching will be honoured. */
+/* If it is a straight lookup, do a lookup for the whole address. This may be
+the empty address. Partial matching doesn't make sense here, so we ignore it,
+but write a panic log entry. However, *@ matching will be honoured. */
 
 if (*s == ';')
   {
@@ -851,9 +960,19 @@ if (*s == ';')
     log_write(0, LOG_MAIN|LOG_PANIC, "partial matching is not applicable to "
       "whole-address lookups: ignored \"partial-\" in \"%s\"", pattern);
   return match_check_string(subject, pattern, -1, FALSE, cb->caseless, FALSE,
-    NULL);
+    valueptr);
   }
 
+/* For the remaining cases, an empty subject matches only an empty pattern,
+because other patterns expect to have a local part and a domain to match
+against. */
+
+if (*subject == 0) return (*pattern == 0)? OK : FAIL;
+
+/* Find the subject's domain */
+
+sdomain = Ustrrchr(subject, '@');
+
 /* If the pattern starts with "@@" we have a split lookup, where the domain is
 looked up to obtain a list of local parts. If the subject's local part is just
 "*" (called from retry) the match always fails. */
@@ -906,7 +1025,8 @@ if (pattern[0] == '@' && pattern[1] == '@')
       else local_yield = OK;
 
       *sdomain = 0;
-      rc = match_check_string(subject, ss, -1, TRUE, cb->caseless, FALSE, NULL);
+      rc = match_check_string(subject, ss, -1, TRUE, cb->caseless, FALSE,
+        valueptr);
       *sdomain = '@';
 
       switch(rc)
@@ -1019,15 +1139,15 @@ listptr = (pdomain == NULL)? pattern : pdomain + 1;
 if (valueptr != NULL) *valueptr = NULL;
 
 return match_check_list(
-  &listptr,              /* list of one item */
-  UCHAR_MAX + 2,         /* impossible separator; don't expand */
-  &domainlist_anchor,    /* it's a domain list */
-  &null,                 /* ptr to NULL means no caching */
-  check_string,          /* the function to do one test */
-  &csb,                  /* its data */
-  MCL_DOMAIN,            /* type of list */
-  csb.subject,           /* string for messages */
-  valueptr);             /* where to pass back lookup data */
+  &listptr,                  /* list of one item */
+  UCHAR_MAX+1,               /* impossible separator; single item */
+  &domainlist_anchor,        /* it's a domain list */
+  &null,                     /* ptr to NULL means no caching */
+  check_string,              /* the function to do one test */
+  &csb,                      /* its data */
+  MCL_DOMAIN + MCL_NOEXPAND, /* domain list; don't expand */
+  csb.subject,               /* string for messages */
+  valueptr);                 /* where to pass back lookup data */
 }
 
 
@@ -1052,6 +1172,7 @@ callback to check_address(). During caseless matching, it will recognize
 Arguments:
   address         address to test
   caseless        TRUE to start in caseless state
+  expand          TRUE to allow list expansion
   listptr         list to check against
   cache_bits      points to cache bits for named address lists, or NULL
   expand_setup    controls setting up of $n variables - passed through
@@ -1059,15 +1180,17 @@ Arguments:
   sep             separator character for the list;
                   may be 0 to get separator from the list;
                   may be UCHAR_MAX+1 for one-item list
+  valueptr        where to return a lookup value, or NULL
 
 Returns:          OK    for a positive match, or end list after a negation;
                   FAIL  for a negative match, or end list after non-negation;
                   DEFER if a lookup deferred
 */
 
-BOOL
-match_address_list(uschar *address, BOOL caseless,
-  uschar **listptr, unsigned int *cache_bits, int expand_setup, int sep)
+int
+match_address_list(uschar *address, BOOL caseless, BOOL expand,
+  uschar **listptr, unsigned int *cache_bits, int expand_setup, int sep,
+  uschar **valueptr)
 {
 uschar *p;
 check_address_block ab;
@@ -1107,7 +1230,8 @@ ab.expand_setup = expand_setup;
 ab.caseless = caseless;
 
 return match_check_list(listptr, sep, &addresslist_anchor, &local_cache_bits,
-  check_address, &ab, MCL_ADDRESS, address, NULL);
+  check_address, &ab, MCL_ADDRESS + (expand? 0:MCL_NOEXPAND), address,
+    valueptr);
 }
 
 /* End of match.c */
