@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/rda.c,v 1.8 2005/06/27 14:29:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code for extracting addresses from a forwarding list
@@ -297,22 +299,17 @@ if (fread(filebuf, 1, statbuf.st_size, fwd) != statbuf.st_size)
   }
 filebuf[statbuf.st_size] = 0;
 
-/* Don't pass statbuf.st_size directly to debug_printf. On some systems it
-is a long, which may not be the same as an int. */
-
 DEBUG(D_route)
-  {
-  int size = (int)statbuf.st_size;
-  debug_printf("%d bytes read from %s\n", size, filename);
-  }
+  debug_printf(OFF_T_FMT " bytes read from %s\n", statbuf.st_size, filename);
 
+(void)fclose(fwd);
 return filebuf;
 
 /* Return an error: the string is already set up. */
 
 ERROR_RETURN:
 *yield = FF_ERROR;
-fclose(fwd);
+(void)fclose(fwd);
 return NULL;
 }
 
@@ -326,25 +323,30 @@ return NULL;
 forwarding list, or to run a filter file and get addresses from there.
 
 Arguments:
-  rdata               the redirection block
-  options             the options bits
-  include_directory   restrain to this directory
-  generated           where to hang generated addresses
-  error               for error messages
-  eblockp             for details of skipped syntax errors (NULL => no skip)
-  filtertype          set to the filter type:
-                        FILTER_FORWARD => a traditional .forward file
-                        FILTER_EXIM    => an Exim filter file
-                        FILTER_SIEVE   => a Sieve filter file
-                      a system filter is always forced to be FILTER_EXIM
+  rdata                     the redirection block
+  options                   the options bits
+  include_directory         restrain to this directory
+  sieve_vacation_directory  passed to sieve_interpret
+  sieve_useraddress         passed to sieve_interpret
+  sieve_subaddress          passed to sieve_interpret
+  generated                 where to hang generated addresses
+  error                     for error messages
+  eblockp                   for details of skipped syntax errors
+                              (NULL => no skip)
+  filtertype                set to the filter type:
+                              FILTER_FORWARD => a traditional .forward file
+                              FILTER_EXIM    => an Exim filter file
+                              FILTER_SIEVE   => a Sieve filter file
+                            a system filter is always forced to be FILTER_EXIM
 
-Returns:              a suitable return for rda_interpret()
+Returns:                    a suitable return for rda_interpret()
 */
 
 static int
 rda_extract(redirect_block *rdata, int options, uschar *include_directory,
-  address_item **generated, uschar **error, error_block **eblockp,
-  int *filtertype)
+  uschar *sieve_vacation_directory, uschar *sieve_useraddress,
+  uschar *sieve_subaddress, address_item **generated, uschar **error,
+  error_block **eblockp, int *filtertype)
 {
 uschar *data;
 
@@ -359,8 +361,9 @@ else data = rdata->string;
 *filtertype = system_filtering? FILTER_EXIM : rda_is_filter(data);
 
 /* Filter interpretation is done by a general function that is also called from
-the filter testing option (-bf). Several features of string expansion may be
-locked out at sites that don't trust users. This is done by setting flags in
+the filter testing option (-bf). There are two versions: one for Exim filtering
+and one for Sieve filtering. Several features of string expansion may be locked
+out at sites that don't trust users. This is done by setting flags in
 expand_forbid that the expander inspects. */
 
 if (*filtertype != FILTER_FORWARD)
@@ -368,20 +371,43 @@ if (*filtertype != FILTER_FORWARD)
   int frc;
   int old_expand_forbid = expand_forbid;
 
+  DEBUG(D_route) debug_printf("data is %s filter program\n",
+    (*filtertype == FILTER_EXIM)? "an Exim" : "a Sieve");
+
+  /* RDO_FILTER is an "allow" bit */
+
   if ((options & RDO_FILTER) == 0)
     {
     *error = US"filtering not enabled";
     return FF_ERROR;
     }
 
-  DEBUG(D_route) debug_printf("data is %s filter program\n",
-    (*filtertype == FILTER_EXIM)? "an Exim" : "a Sieve");
-
   expand_forbid =
     (expand_forbid & ~RDO_FILTER_EXPANSIONS) |
     (options & RDO_FILTER_EXPANSIONS);
-  frc = ((*filtertype == FILTER_EXIM)? filter_interpret : sieve_interpret)
-    (data, options, generated, error);
+
+  /* RDO_{EXIM,SIEVE}_FILTER are forbid bits */
+
+  if (*filtertype == FILTER_EXIM)
+    {
+    if ((options & RDO_EXIM_FILTER) != 0)
+      {
+      *error = US"Exim filtering not enabled";
+      return FF_ERROR;
+      }
+    frc = filter_interpret(data, options, generated, error);
+    }
+  else
+    {
+    if ((options & RDO_SIEVE_FILTER) != 0)
+      {
+      *error = US"Sieve filtering not enabled";
+      return FF_ERROR;
+      }
+    frc = sieve_interpret(data, options, sieve_vacation_directory,
+      sieve_useraddress, sieve_subaddress, generated, error);
+    }
+
   expand_forbid = old_expand_forbid;
   return frc;
   }
@@ -420,8 +446,8 @@ static void
 rda_write_string(int fd, uschar *s)
 {
 int len = (s == NULL)? 0 : Ustrlen(s) + 1;
-write(fd, &len, sizeof(int));
-if (s != NULL) write(fd, s, len);
+(void)write(fd, &len, sizeof(int));
+if (s != NULL) (void)write(fd, s, len);
 }
 
 
@@ -481,21 +507,24 @@ sub-process is NOT used when:
     process (which is usually running as root).
 
 Arguments:
-  rdata              redirect data (file + constraints, or data string)
-  options            options to pass to the extraction functions,
-                       plus ENOTDIR and EACCES handling bits
-  include_directory  restrain :include: to this directory
-  ugid               uid/gid to run under - if NULL, no change
-  generated          where to hang generated addresses, initially NULL
-  error              pointer for error message
-  eblockp            for skipped syntax errors; NULL if no skipping
-  filtertype         set to the type of file:
-                       FILTER_FORWARD => traditional .forward file
-                       FILTER_EXIM    => an Exim filter file
-                       FILTER_SIEVE   => a Sieve filter file
-                     a system filter is always forced to be FILTER_EXIM
-  rname              router name for error messages in the format
-                       "xxx router" or "system filter"
+  rdata                     redirect data (file + constraints, or data string)
+  options                   options to pass to the extraction functions,
+                              plus ENOTDIR and EACCES handling bits
+  include_directory         restrain :include: to this directory
+  sieve_vacation_directory  directory passed to sieve_interpret()
+  sieve_useraddress         passed to sieve_interpret
+  sieve_subaddress          passed to sieve_interpret
+  ugid                      uid/gid to run under - if NULL, no change
+  generated                 where to hang generated addresses, initially NULL
+  error                     pointer for error message
+  eblockp                   for skipped syntax errors; NULL if no skipping
+  filtertype                set to the type of file:
+                              FILTER_FORWARD => traditional .forward file
+                              FILTER_EXIM    => an Exim filter file
+                              FILTER_SIEVE   => a Sieve filter file
+                            a system filter is always forced to be FILTER_EXIM
+  rname                     router name for error messages in the format
+                              "xxx router" or "system filter"
 
 Returns:        values from extraction function, or FF_NONEXIST:
                   FF_DELIVERED     success, a significant action was taken
@@ -511,13 +540,16 @@ Returns:        values from extraction function, or FF_NONEXIST:
 
 int
 rda_interpret(redirect_block *rdata, int options, uschar *include_directory,
-  ugid_block *ugid, address_item **generated, uschar **error,
-  error_block **eblockp, int *filtertype, uschar *rname)
+  uschar *sieve_vacation_directory, uschar *sieve_useraddress,
+  uschar *sieve_subaddress, ugid_block *ugid, address_item **generated,
+  uschar **error, error_block **eblockp, int *filtertype, uschar *rname)
 {
 int fd, rc, pfd[2];
 int yield, status;
+BOOL had_disaster = FALSE;
 pid_t pid;
 uschar *data;
+uschar *readerror = US"";
 void (*oldsignal)(int);
 
 DEBUG(D_route) debug_printf("rda_interpret (%s): %s\n",
@@ -553,8 +585,9 @@ if (!ugid->uid_set ||                         /* Either there's no uid, or */
      rda_is_filter(data) == FILTER_FORWARD && /* It's not a filter script, */
      Ustrstr(data, ":include:") == NULL))     /* and there's no :include: */
   {
-  return rda_extract(rdata, options, include_directory, generated, error,
-    eblockp, filtertype);
+  return rda_extract(rdata, options, include_directory,
+    sieve_vacation_directory, sieve_useraddress, sieve_subaddress,
+    generated, error, eblockp, filtertype);
   }
 
 /* We need to run the processing code in a sub-process. However, if we can
@@ -582,7 +615,7 @@ if ((pid = fork()) == 0)
   header_line *waslast = header_last;   /* Save last header */
 
   fd = pfd[pipe_write];
-  close(pfd[pipe_read]);
+  (void)close(pfd[pipe_read]);
   exim_setugid(ugid->uid, ugid->gid, FALSE, rname);
 
   /* Addresses can get rewritten in filters; if we are not root or the exim
@@ -593,19 +626,20 @@ if ((pid = fork()) == 0)
     {
     DEBUG(D_rewrite) debug_printf("turned off address rewrite logging (not "
       "root or exim in this process)\n");
-    log_selector &= ~L_address_rewrite;
+    log_write_selector &= ~L_address_rewrite;
     }
 
   /* Now do the business */
 
-  yield = rda_extract(rdata, options, include_directory, generated, error,
-    eblockp, filtertype);
+  yield = rda_extract(rdata, options, include_directory,
+    sieve_vacation_directory, sieve_useraddress, sieve_subaddress, generated,
+    error, eblockp, filtertype);
 
   /* Pass back whether it was a filter, and the return code and any overall
   error text via the pipe. */
 
-  write(fd, filtertype, sizeof(int));
-  write(fd, &yield, sizeof(int));
+  (void)write(fd, filtertype, sizeof(int));
+  (void)write(fd, &yield, sizeof(int));
   rda_write_string(fd, *error);
 
   /* Pass back the contents of any syntax error blocks if we have a pointer */
@@ -631,10 +665,10 @@ if ((pid = fork()) == 0)
     header_line *h;
     for (h = header_list; h != waslast->next; i++, h = h->next)
       {
-      if (h->type == htype_old) write(fd, &i, sizeof(i));
+      if (h->type == htype_old) (void)write(fd, &i, sizeof(i));
       }
     i = -1;
-    write(fd, &i, sizeof(i));
+    (void)write(fd, &i, sizeof(i));
 
     while (waslast != header_last)
       {
@@ -642,12 +676,16 @@ if ((pid = fork()) == 0)
       if (waslast->type != htype_old)
         {
         rda_write_string(fd, waslast->text);
-        write(fd, &(waslast->type), sizeof(waslast->type));
+        (void)write(fd, &(waslast->type), sizeof(waslast->type));
         }
       }
     rda_write_string(fd, NULL);    /* Indicates end of added headers */
     }
 
+  /* Write the contents of the $n variables */
+
+  (void)write(fd, filter_n, sizeof(filter_n));
+
   /* If the result was DELIVERED or NOTDELIVERED, we pass back the generated
   addresses, and their associated information, through the pipe. This is
   just tedious, but it seems to be the only safe way. We do this also for
@@ -663,8 +701,8 @@ if ((pid = fork()) == 0)
       int reply_options = 0;
 
       rda_write_string(fd, addr->address);
-      write(fd, &(addr->mode), sizeof(addr->mode));
-      write(fd, &(addr->flags), sizeof(addr->flags));
+      (void)write(fd, &(addr->mode), sizeof(addr->mode));
+      (void)write(fd, &(addr->flags), sizeof(addr->flags));
       rda_write_string(fd, addr->p.errors_address);
 
       if (addr->pipe_expandn != NULL)
@@ -676,15 +714,15 @@ if ((pid = fork()) == 0)
       rda_write_string(fd, NULL);
 
       if (addr->reply == NULL)
-        write(fd, &reply_options, sizeof(int));    /* 0 means no reply */
+        (void)write(fd, &reply_options, sizeof(int));    /* 0 means no reply */
       else
         {
         reply_options |= REPLY_EXISTS;
         if (addr->reply->file_expand) reply_options |= REPLY_EXPAND;
         if (addr->reply->return_message) reply_options |= REPLY_RETURN;
-        write(fd, &reply_options, sizeof(int));
-        write(fd, &(addr->reply->expand_forbid), sizeof(int));
-        write(fd, &(addr->reply->once_repeat), sizeof(time_t));
+        (void)write(fd, &reply_options, sizeof(int));
+        (void)write(fd, &(addr->reply->expand_forbid), sizeof(int));
+        (void)write(fd, &(addr->reply->once_repeat), sizeof(time_t));
         rda_write_string(fd, addr->reply->to);
         rda_write_string(fd, addr->reply->cc);
         rda_write_string(fd, addr->reply->bcc);
@@ -704,8 +742,8 @@ if ((pid = fork()) == 0)
 
   /* OK, this process is now done. Must use _exit() and not exit() !! */
 
-  close(fd);
-  _exit(yield);
+  (void)close(fd);
+  _exit(0);
   }
 
 /* Back in the main process: panic if the fork did not succeed. */
@@ -717,7 +755,7 @@ if (pid < 0)
 writing end must be closed first, as otherwise read() won't return zero on an
 empty pipe. Afterwards, close the reading end. */
 
-close(pfd[pipe_write]);
+(void)close(pfd[pipe_write]);
 
 /* Read initial data, including yield and contents of *error */
 
@@ -767,7 +805,7 @@ if (system_filtering)
       {
       hn++;
       h = h->next;
-      if (h == NULL) goto DISASTER;
+      if (h == NULL) goto DISASTER_NO_HEADER;
       }
     h->type = htype_old;
     }
@@ -783,6 +821,10 @@ if (system_filtering)
     }
   }
 
+/* Read the values of the $n variables */
+
+if (read(fd, filter_n, sizeof(filter_n)) != sizeof(filter_n)) goto DISASTER;
+
 /* If the yield is DELIVERED, NOTDELIVERED, FAIL, or FREEZE there may follow
 addresses and data to go with them. Keep them in the same order in the
 generated chain. */
@@ -877,20 +919,46 @@ while ((rc = wait(&status)) != pid)
   if (rc < 0 && errno == ECHILD)      /* Process has vanished */
     {
     log_write(0, LOG_MAIN, "redirection process %d vanished unexpectedly", pid);
-    break;
+    goto FINAL_EXIT;
     }
   }
 
-close(fd);
+if (had_disaster)
+  {
+  *error = string_sprintf("internal problem in %s: failure to transfer "
+    "data from subprocess: status=%04x%s%s%s", rname,
+    status, readerror,
+    (*error == NULL)? US"" : US": error=",
+    (*error == NULL)? US"" : *error);
+  log_write(0, LOG_MAIN|LOG_PANIC, "%s", *error);
+  }
+else if (status != 0)
+  {
+  log_write(0, LOG_MAIN|LOG_PANIC, "internal problem in %s: unexpected status "
+    "%04x from redirect subprocess (but data correctly received)", rname,
+    status);
+  }
+
+FINAL_EXIT:
+(void)close(fd);
 signal(SIGCHLD, oldsignal);   /* restore */
 return yield;
 
-/* Come here is there's a shambles in transferring the data over the pipe. */
+
+/* Come here if the data indicates removal of a header that we can't find */
+
+DISASTER_NO_HEADER:
+readerror = US" readerror=bad header identifier";
+had_disaster = TRUE;
+yield = FF_ERROR;
+goto WAIT_EXIT;
+
+/* Come here is there's a shambles in transferring the data over the pipe. The
+value of errno should still be set. */
 
 DISASTER:
-*error = string_sprintf("internal problem in %s: failure to transfer "
-  "data from subprocess", rname);
-log_write(0, LOG_MAIN|LOG_PANIC, "%s", *error);
+readerror = string_sprintf(" readerror='%s'", strerror(errno));
+had_disaster = TRUE;
 yield = FF_ERROR;
 goto WAIT_EXIT;
 }
