@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/routers/dnslookup.h,v 1.2 2005/01/04 10:00:44 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -14,6 +16,9 @@ typedef struct {
   BOOL rewrite_headers;
   uschar *widen_domains;
   uschar *mx_domains;
+  uschar *mx_fail_domains;
+  uschar *srv_fail_domains;
+  uschar *check_srv;
 } dnslookup_router_options_block;
 
 /* Data for reading the private options. */
