@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/sieve.c,v 1.12 2005/06/17 10:47:05 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) Michael Haardt 2003 */
+/* Copyright (c) Michael Haardt 2003-2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This code was contributed by Michael Haardt. */
@@ -27,8 +29,15 @@
 #undef RFC_EOL
 
 /* Define this for development of the subaddress Sieve extension.   */
-/* The code is currently broken.                                    */
-#undef SUBADDRESS
+#define SUBADDRESS
+
+/* Define this for the vacation Sieve extension.                    */
+#define VACATION
+
+/* Must be >= 1                                                     */
+#define VACATION_MIN_DAYS 1
+/* Must be >= VACATION_MIN_DAYS, must be > 7, should be > 30        */
+#define VACATION_MAX_DAYS 31
 
 /* Keep this at 75 to accept only RFC compliant MIME words.         */
 /* Increase it if you want to match headers from buggy MUAs.        */
@@ -46,11 +55,25 @@ struct Sieve
 #ifdef SUBADDRESS
   int require_subaddress;
 #endif
+#ifdef VACATION
+  int require_vacation;
+  int vacation_ran;
+#endif
+  uschar *vacation_directory;
+  const uschar *subaddress;
+  const uschar *useraddress;
+  int require_copy;
+  int require_iascii_numeric;
   };
 
-enum Comparator { COMP_OCTET, COMP_ASCII_CASEMAP };
+enum Comparator { COMP_OCTET, COMP_EN_ASCII_CASEMAP, COMP_ASCII_NUMERIC };
 enum MatchType { MATCH_IS, MATCH_CONTAINS, MATCH_MATCHES };
+#ifdef SUBADDRESS
 enum AddressPart { ADDRPART_USER, ADDRPART_DETAIL, ADDRPART_LOCALPART, ADDRPART_DOMAIN, ADDRPART_ALL };
+#else
+enum AddressPart { ADDRPART_LOCALPART, ADDRPART_DOMAIN, ADDRPART_ALL };
+#endif
+enum RelOp { LT, LE, EQ, GE, GT, NE };
 
 struct String
   {
@@ -83,14 +106,135 @@ static const struct String str_envelope={ str_envelope_c, 8 };
 static uschar str_subaddress_c[]="subaddress";
 static const struct String str_subaddress={ str_subaddress_c, 10 };
 #endif
+#ifdef VACATION
+static uschar str_vacation_c[]="vacation";
+static const struct String str_vacation={ str_vacation_c, 8 };
+static uschar str_subject_c[]="Subject";
+static const struct String str_subject={ str_subject_c, 7 };
+#endif
+static uschar str_copy_c[]="copy";
+static const struct String str_copy={ str_copy_c, 4 };
 static uschar str_iascii_casemap_c[]="i;ascii-casemap";
 static const struct String str_iascii_casemap={ str_iascii_casemap_c, 15 };
+static uschar str_enascii_casemap_c[]="en;ascii-casemap";
+static const struct String str_enascii_casemap={ str_enascii_casemap_c, 16 };
 static uschar str_ioctet_c[]="i;octet";
 static const struct String str_ioctet={ str_ioctet_c, 7 };
+static uschar str_iascii_numeric_c[]="i;ascii-numeric";
+static const struct String str_iascii_numeric={ str_iascii_numeric_c, 15 };
 static uschar str_comparator_iascii_casemap_c[]="comparator-i;ascii-casemap";
 static const struct String str_comparator_iascii_casemap={ str_comparator_iascii_casemap_c, 26 };
+static uschar str_comparator_enascii_casemap_c[]="comparator-en;ascii-casemap";
+static const struct String str_comparator_enascii_casemap={ str_comparator_enascii_casemap_c, 27 };
 static uschar str_comparator_ioctet_c[]="comparator-i;octet";
 static const struct String str_comparator_ioctet={ str_comparator_ioctet_c, 18 };
+static uschar str_comparator_iascii_numeric_c[]="comparator-i;ascii-numeric";
+static const struct String str_comparator_iascii_numeric={ str_comparator_iascii_numeric_c, 26 };
+
+
+/*************************************************
+*          Encode to quoted-printable            *
+*************************************************/
+
+/*
+Arguments:
+  src               UTF-8 string
+*/
+
+static struct String *quoted_printable_encode(const struct String *src, struct String *dst)
+{
+int pass;
+const uschar *start,*end;
+uschar *new = NULL;
+uschar ch;
+size_t line;
+
+for (pass=0; pass<=1; ++pass)
+  {
+  line=0;
+  if (pass==0)
+    dst->length=0;
+  else
+    {
+    dst->character=store_get(dst->length+1); /* plus one for \0 */
+    new=dst->character;
+    }
+  for (start=src->character,end=start+src->length; start<end; ++start)
+    {
+    ch=*start;
+    if (line>=73)
+      {
+      if (pass==0)
+        dst->length+=2;
+      else
+        {
+        *new++='=';
+        *new++='\n';
+        }
+      line=0;
+      }
+    if
+      (
+      (ch>=33 && ch<=60)
+      || (ch>=62 && ch<=126)
+      ||
+        (
+        (ch==9 || ch==32)
+#ifdef RFC_EOL
+        && start+2<end
+        && (*(start+1)!='\r' || *(start+2)!='\n')
+#else
+        && start+1<end
+        && *(start+1)!='\n'
+#endif
+        )
+      )
+      {
+      if (pass==0)
+        ++dst->length;
+      else
+        *new++=*start;
+      ++line;
+      }
+#ifdef RFC_EOL
+    else if (ch=='\r' && start+1<end && *(start+1)=='\n')
+      {
+      if (pass==0)
+        {
+        ++dst->length;
+        line=0;
+        ++start;
+        }
+      else
+        *new++='\n';
+        line=0;
+      }
+#else
+    else if (ch=='\n')
+      {
+      if (pass==0)
+        ++dst->length;
+      else
+        *new++=*start;
+      ++line;
+      }
+#endif
+    else
+      {
+      if (pass==0)
+        dst->length+=3;
+      else
+        {
+        sprintf(CS new,"=%02X",ch);
+        new+=3;
+        }
+      line+=3;
+      }
+    }
+  }
+  *new='\0'; /* not included in length, but nice */
+  return dst;
+}
 
 
 /*************************************************
@@ -417,6 +561,59 @@ return (h.length==0);
 }
 
 
+/*************************************************
+*    ASCII numeric comparison                    *
+*************************************************/
+
+/*
+Arguments:
+  a                 first numeric string
+  b                 second numeric string
+  relop             relational operator
+
+Returns:      0               not (a relop b)
+              1               a relop b
+*/
+
+static int eq_asciinumeric(const struct String *a,
+  const struct String *b, enum RelOp relop)
+{
+size_t al,bl;
+const uschar *as,*aend,*bs,*bend;
+int cmp;
+
+as=a->character;
+aend=a->character+a->length;
+bs=b->character;
+bend=b->character+b->length;
+
+while (*as>='0' && *as<='9' && as<aend) ++as;
+al=as-a->character;
+while (*bs>='0' && *bs<='9' && bs<bend) ++bs;
+bl=bs-b->character;
+
+if (al && bl==0) cmp=-1;
+else if (al==0 && bl==0) cmp=0;
+else if (al==0 && bl) cmp=1;
+else
+  {
+  cmp=al-bl;
+  if (cmp==0) cmp=memcmp(a->character,b->character,al);
+  }
+switch (relop)
+  {
+  case LT: return cmp<0;
+  case LE: return cmp<=0;
+  case EQ: return cmp==0;
+  case GE: return cmp>=0;
+  case GT: return cmp>0;
+  case NE: return cmp!=0;
+  }
+  /*NOTREACHED*/
+  return -1;
+}
+
+
 /*************************************************
 *             Compare strings                    *
 *************************************************/
@@ -430,14 +627,15 @@ Arguments:
 
 Returns:      0               needle not found in haystack
               1               needle found
+              -1              comparator does not offer matchtype
 */
 
-static int compare(const struct String *needle, const struct String *haystack,
+static int compare(struct Sieve *filter, const struct String *needle, const struct String *haystack,
   enum Comparator co, enum MatchType mt)
 {
 int r=0;
 
-if ((filter_test != NULL && debug_selector != 0) ||
+if ((filter_test != FTEST_NONE && debug_selector != 0) ||
   (debug_selector & D_filter) != 0)
   {
   debug_printf("String comparison (match ");
@@ -451,7 +649,8 @@ if ((filter_test != NULL && debug_selector != 0) ||
   switch (co)
     {
     case COMP_OCTET: debug_printf("i;octet"); break;
-    case COMP_ASCII_CASEMAP: debug_printf("i;ascii-casemap"); break;
+    case COMP_EN_ASCII_CASEMAP: debug_printf("en;ascii-casemap"); break;
+    case COMP_ASCII_NUMERIC: debug_printf("i;ascii-numeric"); break;
     }
   debug_printf("\"):\n");
   debug_printf("  Search = %s (%d chars)\n", needle->character,needle->length);
@@ -468,11 +667,21 @@ switch (mt)
         if (eq_octet(needle,haystack,0)) r=1;
         break;
         }
-      case COMP_ASCII_CASEMAP:
+      case COMP_EN_ASCII_CASEMAP:
         {
         if (eq_asciicase(needle,haystack,0)) r=1;
         break;
         }
+      case COMP_ASCII_NUMERIC:
+        {
+        if (!filter->require_iascii_numeric)
+          {
+          filter->errmsg=CUS "missing previous require \"comparator-i;ascii-numeric\";";
+          return -1;
+          }
+        if (eq_asciinumeric(needle,haystack,EQ)) r=1;
+        break;
+        }
       }
     break;
     }
@@ -487,11 +696,16 @@ switch (mt)
         for (h=*haystack; h.length; ++h.character,--h.length) if (eq_octet(needle,&h,1)) { r=1; break; }
         break;
         }
-      case COMP_ASCII_CASEMAP:
+      case COMP_EN_ASCII_CASEMAP:
         {
         for (h=*haystack; h.length; ++h.character,--h.length) if (eq_asciicase(needle,&h,1)) { r=1; break; }
         break;
         }
+      default:
+        {
+        filter->errmsg=CUS "comparator does not offer specified matchtype";
+        return -1;
+        }
       }
     break;
     }
@@ -504,16 +718,21 @@ switch (mt)
         if (eq_octetglob(needle,haystack)) r=1;
         break;
         }
-      case COMP_ASCII_CASEMAP:
+      case COMP_EN_ASCII_CASEMAP:
         {
         if (eq_asciicaseglob(needle,haystack)) r=1;
         break;
         }
+      default:
+        {
+        filter->errmsg=CUS "comparator does not offer specified matchtype";
+        return -1;
+        }
       }
     break;
     }
   }
-if ((filter_test != NULL && debug_selector != 0) ||
+if ((filter_test != FTEST_NONE && debug_selector != 0) ||
   (debug_selector & D_filter) != 0)
   debug_printf("  Result %s\n",r?"true":"false");
 return r;
@@ -629,7 +848,7 @@ Arguments:
 Returns:      nothing
 */
 
-static void add_addr(address_item **generated, uschar *addr, int file)
+static void add_addr(address_item **generated, uschar *addr, int file, int maxage, int maxmessages, int maxstorage)
 {
 address_item *new_addr;
 
@@ -637,7 +856,7 @@ for (new_addr=*generated; new_addr; new_addr=new_addr->next)
   {
   if (Ustrcmp(new_addr->address,addr)==0 && (file ? testflag(new_addr, af_pfr|af_file) : 1))
     {
-    if ((filter_test != NULL && debug_selector != 0) || (debug_selector & D_filter) != 0)
+    if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
       {
       debug_printf("Repeated %s `%s' ignored.\n",file ? "fileinto" : "redirect", addr);
       }
@@ -645,7 +864,7 @@ for (new_addr=*generated; new_addr; new_addr=new_addr->next)
     }
   }
 
-if ((filter_test != NULL && debug_selector != 0) || (debug_selector & D_filter) != 0)
+if ((filter_test != FTEST_NONE && debug_selector != 0) || (debug_selector & D_filter) != 0)
   {
   debug_printf("%s `%s'\n",file ? "fileinto" : "redirect", addr);
   }
@@ -1109,6 +1328,7 @@ if (*filter->pc=='[') /* string list */
   if (*filter->pc==']')
     {
     d[dataLength].character=(uschar*)0;
+    d[dataLength].length=-1;
     ++filter->pc;
     *data=d;
     return 1;
@@ -1138,6 +1358,7 @@ else /* single string */
   else
     {
     d[1].character=(uschar*)0;
+    d[1].length=-1;
     *data=d;
     return 1;
     }
@@ -1154,7 +1375,8 @@ return -1;
 
 /*
 Grammar:
-  address-part = ":user" / ":detail" / ":localpart" / ":domain" / ":all"
+  address-part     =  ":localpart" / ":domain" / ":all"
+  address-part     =/ ":user" / ":detail"
 
 Arguments:
   filter      points to the Sieve filter including its state
@@ -1223,6 +1445,7 @@ Arguments:
 
 Returns:      1                success
               0                no comparator found
+              -1               incomplete comparator found
 */
 
 static int parse_comparator(struct Sieve *filter, enum Comparator *c)
@@ -1250,7 +1473,17 @@ switch (parse_string(filter,&comparator_name))
       }
     else if (eq_asciicase(&comparator_name,&str_iascii_casemap,0))
       {
-      *c=COMP_ASCII_CASEMAP;
+      *c=COMP_EN_ASCII_CASEMAP;
+      match=1;
+      }
+    else if (eq_asciicase(&comparator_name,&str_enascii_casemap,0))
+      {
+      *c=COMP_EN_ASCII_CASEMAP;
+      match=1;
+      }
+    else if (eq_asciicase(&comparator_name,&str_iascii_numeric,0))
+      {
+      *c=COMP_ASCII_NUMERIC;
       match=1;
       }
     else
@@ -1385,7 +1618,7 @@ if (parse_identifier(filter,CUS "address"))
   */
 
   enum AddressPart addressPart=ADDRPART_ALL;
-  enum Comparator comparator=COMP_ASCII_CASEMAP;
+  enum Comparator comparator=COMP_EN_ASCII_CASEMAP;
   enum MatchType matchType=MATCH_IS;
   struct String *hdr,*h,*key,*k;
   int m;
@@ -1439,7 +1672,7 @@ if (parse_identifier(filter,CUS "address"))
     return -1;
     }
   *cond=0;
-  for (h=hdr; h->character && !*cond; ++h)
+  for (h=hdr; h->length!=-1 && !*cond; ++h)
     {
     uschar *header_value=(uschar*)0,*extracted_addr,*end_addr;
 
@@ -1479,38 +1712,34 @@ if (parse_identifier(filter,CUS "address"))
         *end_addr = 0;
         extracted_addr = parse_extract_address(header_value, &error, &start, &end, &domain, FALSE);
 
-        /* Modification by PH: clever compilers complain when switching on an
-        enumeration if not all the possible values appear. So we have to have
-        them all; just do nothing when nothing need be done. */
-
         if (extracted_addr) switch (addressPart)
           {
           case ADDRPART_ALL: part=extracted_addr; break;
-
+#ifdef SUBADDRESS
           case ADDRPART_USER:
-#ifndef SUBADDRESS
-          break;    /* Else fall through */
 #endif
           case ADDRPART_LOCALPART: part=extracted_addr; part[domain-1]='\0'; break;
           case ADDRPART_DOMAIN: part=extracted_addr+domain; break;
-
-          case ADDRPART_DETAIL:
 #ifdef SUBADDRESS
-          part=NULL;
+          case ADDRPART_DETAIL: part=NULL; break;
 #endif
-          break;
           }
 
         *end_addr = saveend;
         if (part)
           {
-          for (k=key; k->character; ++k)
+          for (k=key; k->length!=-1; ++k)
             {
             struct String partStr;
 
             partStr.character=part;
             partStr.length=Ustrlen(part);
-            if (extracted_addr && (*cond=compare(k,&partStr,comparator,matchType))) break;
+            if (extracted_addr)
+              {
+              *cond=compare(filter,k,&partStr,comparator,matchType);
+              if (*cond==-1) return -1;
+              if (*cond) break;
+              }
             }
           }
         if (saveend == 0) break;
@@ -1568,7 +1797,7 @@ else if (parse_identifier(filter,CUS "exists"))
   if (exec)
     {
     *cond=1;
-    for (h=hdr; h->character && *cond; ++h)
+    for (h=hdr; h->length!=-1 && *cond; ++h)
       {
       uschar *header_def;
 
@@ -1599,7 +1828,7 @@ else if (parse_identifier(filter,CUS "header"))
                 <header-names: string-list> <key-list: string-list>
   */
 
-  enum Comparator comparator=COMP_ASCII_CASEMAP;
+  enum Comparator comparator=COMP_EN_ASCII_CASEMAP;
   enum MatchType matchType=MATCH_IS;
   struct String *hdr,*h,*key,*k;
   int m;
@@ -1643,7 +1872,7 @@ else if (parse_identifier(filter,CUS "header"))
     return -1;
     }
   *cond=0;
-  for (h=hdr; h->character && !*cond; ++h)
+  for (h=hdr; h->length!=-1 && !*cond; ++h)
     {
     if (!is_header(h))
       {
@@ -1662,9 +1891,14 @@ else if (parse_identifier(filter,CUS "header"))
         filter->errmsg=CUS "header string expansion failed";
         return -1;
         }
-      for (k=key; k->character; ++k)
+      for (k=key; k->length!=-1; ++k)
         {
-        if (Ustrcmp(header_def,"true")==0 && (*cond=compare(k,&header_value,comparator,matchType))) break;
+        if (Ustrcmp(header_def,"true")==0)
+          {
+          *cond=compare(filter,k,&header_value,comparator,matchType);
+          if (*cond==-1) return -1;
+          if (*cond) break;
+          }
         }
       }
     }
@@ -1717,7 +1951,7 @@ else if (parse_identifier(filter,CUS "envelope"))
   envelope-part is case insensitive "from" or "to"
   */
 
-  enum Comparator comparator=COMP_ASCII_CASEMAP;
+  enum Comparator comparator=COMP_EN_ASCII_CASEMAP;
   enum AddressPart addressPart=ADDRPART_ALL;
   enum MatchType matchType=MATCH_IS;
   struct String *env,*e,*key,*k;
@@ -1784,46 +2018,27 @@ else if (parse_identifier(filter,CUS "envelope"))
 
     if (eq_asciicase(e,&str_from,0))
       {
-      /* Modification by PH: clever compilers complain when switching on an
-      enumeration if not all the possible values appear. So we have to have
-      them all; just do nothing when nothing need be done. */
-
       switch (addressPart)
         {
         case ADDRPART_ALL: envelopeExpr=CUS "$sender_address"; break;
-
+#ifdef SUBADDRESS
         case ADDRPART_USER:
-#ifndef SUBADDRESS
-        break;     /* Else fall through */
 #endif
         case ADDRPART_LOCALPART: envelopeExpr=CUS "${local_part:$sender_address}"; break;
         case ADDRPART_DOMAIN: envelopeExpr=CUS "${domain:$sender_address}"; break;
-
-        case ADDRPART_DETAIL:
 #ifdef SUBADDRESS
-        envelopeExpr=CUS 0;
+        case ADDRPART_DETAIL: envelopeExpr=CUS 0; break;
 #endif
-        break;
         }
       }
-
     else if (eq_asciicase(e,&str_to,0))
       {
-      /* Modification by PH: clever compilers complain when switching on an
-      enumeration if not all the possible values appear. So we have to have
-      them all; just do nothing when nothing need be done. */
-
       switch (addressPart)
         {
         case ADDRPART_ALL: envelopeExpr=CUS "$local_part_prefix$local_part$local_part_suffix@$domain"; break;
-
 #ifdef SUBADDRESS
-        case ADDRPART_USER: envelopeExpr=CUS "$local_part_prefix$local_part"; break;
-        case ADDRPART_DETAIL: envelopeExpr=CUS "$local_part_suffix"; break;
-#else
-        case ADDRPART_USER:
-        case ADDRPART_DETAIL:
-        break;
+        case ADDRPART_USER: envelopeExpr=filter->useraddress; break;
+        case ADDRPART_DETAIL: envelopeExpr=filter->subaddress; break;
 #endif
         case ADDRPART_LOCALPART: envelopeExpr=CUS "$local_part_prefix$local_part$local_part_suffix"; break;
         case ADDRPART_DOMAIN: envelopeExpr=CUS "$domain"; break;
@@ -1841,13 +2056,15 @@ else if (parse_identifier(filter,CUS "envelope"))
         filter->errmsg=CUS "header string expansion failed";
         return -1;
         }
-      for (k=key; k->character; ++k)
+      for (k=key; k->length!=-1; ++k)
         {
         struct String envelopeStr;
 
         envelopeStr.character=envelope;
         envelopeStr.length=Ustrlen(envelope);
-        if ((*cond=compare(&envelopeStr,k,comparator,matchType))) break;
+        *cond=compare(filter,k,&envelopeStr,comparator,matchType);
+        if (*cond==-1) return -1;
+        if (*cond) break;
         }
       }
     }
@@ -1890,7 +2107,7 @@ if (*filter->pc=='{')
     }
   else
     {
-    filter->errmsg=CUS "missing closing brace";
+    filter->errmsg=CUS "expecting command or closing brace";
     return -1;
     }
   }
@@ -1984,7 +2201,7 @@ while (*filter->pc)
           return -1;
           }
         m=parse_block(filter,exec && unsuccessful ? cond : 0, generated);
-        if (m==-1) return -1;
+        if (m==-1 || m==2) return m;
         if (m==0)
           {
           filter->errmsg=CUS "missing block";
@@ -2010,7 +2227,8 @@ while (*filter->pc)
   else if (parse_identifier(filter,CUS "stop"))
     {
     /*
-    stop-command = "stop" ";"
+    stop-command     =  "stop" { stop-options } ";"
+    stop-options     =
     */
 
     if (parse_semicolon(filter)==-1) return -1;
@@ -2023,20 +2241,22 @@ while (*filter->pc)
   else if (parse_identifier(filter,CUS "keep"))
     {
     /*
-    keep-command = "keep" ";"
+    keep-command     =  "keep" { keep-options } ";"
+    keep-options     =
     */
 
     if (parse_semicolon(filter)==-1) return -1;
     if (exec)
       {
-      add_addr(generated,US"inbox",1);
+      add_addr(generated,US"inbox",1,0,0,0);
       filter->keep = 0;
       }
     }
   else if (parse_identifier(filter,CUS "discard"))
     {
     /*
-    discard-command    = "discard" ";"
+    discard-command  =  "discard" { discard-options } ";"
+    discard-options  =
     */
 
     if (parse_semicolon(filter)==-1) return -1;
@@ -2045,12 +2265,29 @@ while (*filter->pc)
   else if (parse_identifier(filter,CUS "redirect"))
     {
     /*
-    redirect-command = "redirect" "string" ";"
+    redirect-command =  "redirect" redirect-options "string" ";"
+    redirect-options =
+    redirect-options =) ":copy"
     */
 
     struct String recipient;
     int m;
+    int copy=0;
 
+    for (;;)
+      {
+      if (parse_white(filter)==-1) return -1;
+      if (parse_identifier(filter,CUS ":copy")==1)
+        {
+        if (!filter->require_copy)
+          {
+          filter->errmsg=CUS "missing previous require \"copy\";";
+          return -1;
+          }
+          copy=1;
+        }
+      else break;
+      }
     if (parse_white(filter)==-1) return -1;
     if ((m=parse_string(filter,&recipient))!=1)
       {
@@ -2064,26 +2301,45 @@ while (*filter->pc)
       }
     if (exec)
       {
-      add_addr(generated,recipient.character,0);
-      filter->keep = 0;
+      add_addr(generated,recipient.character,0,0,0,0);
+      if (!copy) filter->keep = 0;
       }
     if (parse_semicolon(filter)==-1) return -1;
     }
   else if (parse_identifier(filter,CUS "fileinto"))
     {
     /*
-    fileinto-command   = "fileinto" string ";"
-    */
+    fileinto-command =  "fileinto" { fileinto-options } string ";"
+    fileinto-options =
+    fileinto-options =) [ ":copy" ]
+   */
 
     struct String folder;
     uschar *s;
     int m;
+    unsigned long maxage, maxmessages, maxstorage;
+    int copy=0;
 
+    maxage = maxmessages = maxstorage = 0;
     if (!filter->require_fileinto)
       {
       filter->errmsg=CUS "missing previous require \"fileinto\";";
       return -1;
       }
+    for (;;)
+      {
+      if (parse_white(filter)==-1) return -1;
+      if (parse_identifier(filter,CUS ":copy")==1)
+        {
+        if (!filter->require_copy)
+          {
+          filter->errmsg=CUS "missing previous require \"copy\";";
+          return -1;
+          }
+          copy=1;
+        }
+      else break;
+      }
     if (parse_white(filter)==-1) return -1;
     if ((m=parse_string(filter,&folder))!=1)
       {
@@ -2105,12 +2361,291 @@ while (*filter->pc)
       }
     if (exec)
       {
-      add_addr(generated, folder.character, 1);
-      filter->keep = 0;
+      add_addr(generated, folder.character, 1, maxage, maxmessages, maxstorage);
+      if (!copy) filter->keep = 0;
       }
     if (parse_semicolon(filter)==-1) return -1;
     }
-  else break;
+#ifdef VACATION
+  else if (parse_identifier(filter,CUS "vacation"))
+    {
+    /*
+    vacation-command =  "vacation" { vacation-options } <reason: string> ";"
+    vacation-options =  [":days" number]
+                        [":subject" string]
+                        [":from" string]
+                        [":addresses" string-list]
+                        [":mime"]
+                        [":handle" string]
+    */
+
+    int m;
+    unsigned long days;
+    struct String subject;
+    struct String from;
+    struct String *addresses;
+    int reason_is_mime;
+    string_item *aliases;
+    struct String handle;
+    struct String reason;
+
+    if (!filter->require_vacation)
+      {
+      filter->errmsg=CUS "missing previous require \"vacation\";";
+      return -1;
+      }
+    if (exec)
+      {
+      if (filter->vacation_ran)
+        {
+        filter->errmsg=CUS "trying to execute vacation more than once";
+        return -1;
+        }
+      filter->vacation_ran=1;
+      }
+    days=VACATION_MIN_DAYS>7 ? VACATION_MIN_DAYS : 7;
+    subject.character=(uschar*)0;
+    subject.length=-1;
+    from.character=(uschar*)0;
+    from.length=-1;
+    addresses=(struct String*)0;
+    aliases=NULL;
+    reason_is_mime=0;
+    handle.character=(uschar*)0;
+    handle.length=-1;
+    for (;;)
+      {
+      if (parse_white(filter)==-1) return -1;
+      if (parse_identifier(filter,CUS ":days")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
+        if (parse_number(filter,&days)==-1) return -1;
+        if (days<VACATION_MIN_DAYS) days=VACATION_MIN_DAYS;
+        else if (days>VACATION_MAX_DAYS) days=VACATION_MAX_DAYS;
+        }
+      else if (parse_identifier(filter,CUS ":subject")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&subject))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "subject string expected";
+          return -1;
+          }
+        }
+      else if (parse_identifier(filter,CUS ":from")==1)
+        {
+        int start, end, domain;
+        uschar *error,*ss;
+
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&from))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "from string expected";
+          return -1;
+          }
+        if (from.length>0)
+          {
+          ss = parse_extract_address(from.character, &error, &start, &end, &domain,
+            FALSE);
+          if (ss == NULL)
+            {
+            filter->errmsg=string_sprintf("malformed address \"%s\" in "
+              "Sieve filter: %s", from.character, error);
+            return -1;
+            }
+          }
+        else
+          {
+          filter->errmsg=CUS "empty :from address in Sieve filter";
+          return -1;
+          }
+        }
+      else if (parse_identifier(filter,CUS ":addresses")==1)
+        {
+        struct String *a;
+
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_stringlist(filter,&addresses))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "addresses string list expected";
+          return -1;
+          }
+        for (a=addresses; a->length!=-1; ++a)
+          {
+          string_item *new;
+
+          new=store_get(sizeof(string_item));
+          new->text=store_get(a->length+1);
+          if (a->length) memcpy(new->text,a->character,a->length);
+          new->text[a->length]='\0';
+          new->next=aliases;
+          aliases=new;
+          }
+        }
+      else if (parse_identifier(filter,CUS ":mime")==1)
+        reason_is_mime=1;
+      else if (parse_identifier(filter,CUS ":handle")==1)
+        {
+        if (parse_white(filter)==-1) return -1;
+        if ((m=parse_string(filter,&from))!=1)
+          {
+          if (m==0) filter->errmsg=CUS "handle string expected";
+          return -1;
+          }
+        }
+      else break;
+      }
+    if (parse_white(filter)==-1) return -1;
+    if ((m=parse_string(filter,&reason))!=1)
+      {
+      if (m==0) filter->errmsg=CUS "missing reason string";
+      return -1;
+      }
+    if (reason_is_mime)
+      {
+      uschar *s,*end;
+
+      for (s=reason.character,end=reason.character+reason.length; s<end && (*s&0x80)==0; ++s);
+      if (s<end)
+        {
+        filter->errmsg=CUS "MIME reason string contains 8bit text";
+        return -1;
+        }
+      }
+    if (parse_semicolon(filter)==-1) return -1;
+
+    if (exec)
+      {
+      address_item *addr;
+      int capacity,start;
+      uschar *buffer;
+      int buffer_capacity;
+      struct String key;
+      md5 base;
+      uschar digest[16];
+      uschar hexdigest[33];
+      int i;
+      uschar *once;
+
+      if (filter_personal(aliases,TRUE))
+        {
+        if (filter_test == FTEST_NONE)
+          {
+          /* ensure oncelog directory exists; failure will be detected later */
+
+          (void)directory_make(NULL, filter->vacation_directory, 0700, FALSE);
+          }
+        /* build oncelog filename */
+
+        key.character=(uschar*)0;
+        key.length=0;
+        capacity=0;
+        if (handle.length==-1)
+          {
+          if (subject.length!=-1) key.character=string_cat(key.character,&capacity,&key.length,subject.character,subject.length);
+          if (from.length!=-1) key.character=string_cat(key.character,&capacity,&key.length,from.character,from.length);
+          key.character=string_cat(key.character,&capacity,&key.length,reason_is_mime?US"1":US"0",1);
+          key.character=string_cat(key.character,&capacity,&key.length,reason.character,reason.length);
+          }
+        else
+          key=handle;
+        md5_start(&base);
+        md5_end(&base, key.character, key.length, digest);
+        for (i = 0; i < 16; i++) sprintf(CS (hexdigest+2*i), "%02X", digest[i]);
+        if (filter_test != FTEST_NONE)
+          {
+          debug_printf("Sieve: mail was personal, vacation file basename: %s\n", hexdigest);
+          }
+        else
+          {
+          capacity=Ustrlen(filter->vacation_directory);
+          start=capacity;
+          once=string_cat(filter->vacation_directory,&capacity,&start,US"/",1);
+          once=string_cat(once,&capacity,&start,hexdigest,33);
+          once[start] = '\0';
+
+          /* process subject */
+
+          if (subject.length==-1)
+            {
+            expand_header(&subject,&str_subject);
+            capacity=6;
+            start=6;
+            subject.character=string_cat(US"Auto: ",&capacity,&start,subject.character,subject.length);
+            subject.length=start;
+            }
+
+          /* add address to list of generated addresses */
+
+          addr = deliver_make_addr(string_sprintf(">%.256s", sender_address), FALSE);
+          setflag(addr, af_pfr);
+          setflag(addr, af_ignore_error);
+          addr->next = *generated;
+          *generated = addr;
+          addr->reply = store_get(sizeof(reply_item));
+          memset(addr->reply,0,sizeof(reply_item)); /* XXX */
+          addr->reply->to = string_copy(sender_address);
+          if (from.length==-1)
+            addr->reply->from = expand_string(US"$local_part@$domain");
+          else
+            addr->reply->from = from.character;
+          /* Allocation is larger than neccessary, but enough even for split MIME words */
+          buffer_capacity=16+4*subject.length;
+          buffer=store_get(buffer_capacity);
+          addr->reply->subject=parse_quote_2047(subject.character, subject.length, US"utf-8", buffer, buffer_capacity);
+          addr->reply->oncelog=once;
+          addr->reply->once_repeat=days*86400;
+
+          /* build body and MIME headers */
+
+          if (reason_is_mime)
+            {
+            uschar *mime_body,*reason_end;
+#ifdef RFC_EOL
+            static const uschar nlnl[]="\r\n\r\n";
+#else
+            static const uschar nlnl[]="\n\n";
+#endif
+
+            for
+              (
+              mime_body=reason.character,reason_end=reason.character+reason.length;
+              mime_body<(reason_end-sizeof(nlnl)-1) && memcmp(mime_body,nlnl,sizeof(nlnl)-1);
+              ++mime_body
+              );
+            capacity = 0;
+            start = 0;
+            addr->reply->headers = string_cat(NULL,&capacity,&start,reason.character,mime_body-reason.character);
+            addr->reply->headers[start] = '\0';
+            capacity = 0;
+            start = 0;
+            if (mime_body+(sizeof(nlnl)-1)<reason_end) mime_body+=sizeof(nlnl)-1;
+            else mime_body=reason_end-1;
+            addr->reply->text = string_cat(NULL,&capacity,&start,mime_body,reason_end-mime_body);
+            addr->reply->text[start] = '\0';
+            }
+          else
+            {
+            struct String qp;
+
+            capacity = 0;
+            start = reason.length;
+            addr->reply->headers = US"MIME-Version: 1.0\n"
+                                   "Content-Type: text/plain;\n"
+                                   "\tcharset=\"utf-8\"\n"
+                                   "Content-Transfer-Encoding: quoted-printable";
+            addr->reply->text = quoted_printable_encode(&reason,&qp)->character;
+            }
+          }
+        }
+        else if (filter_test != FTEST_NONE)
+          {
+          debug_printf("Sieve: mail was not personal, vacation would ignore it\n");
+          }
+      }
+    }
+    else break;
+#endif
   }
 return 1;
 }
@@ -2141,8 +2676,49 @@ filter->require_fileinto=0;
 #ifdef SUBADDRESS
 filter->require_subaddress=0;
 #endif
+#ifdef VACATION
+filter->require_vacation=0;
+filter->vacation_ran=0;
+#endif
+filter->require_copy=0;
+filter->require_iascii_numeric=0;
 
 if (parse_white(filter)==-1) return -1;
+
+if (exec && filter->vacation_directory != NULL && filter_test == FTEST_NONE)
+  {
+  DIR *oncelogdir;
+  struct dirent *oncelog;
+  struct stat properties;
+  time_t now;
+
+  /* clean up old vacation log databases */
+
+  oncelogdir=opendir(CS filter->vacation_directory);
+
+  if (oncelogdir ==(DIR*)0 && errno != ENOENT)
+    {
+    filter->errmsg=CUS "unable to open vacation directory";
+    return -1;
+    }
+
+  if (oncelogdir != NULL)
+    {
+    time(&now);
+
+    while ((oncelog=readdir(oncelogdir))!=(struct dirent*)0)
+      {
+      if (strlen(oncelog->d_name)==32)
+        {
+        uschar *s=string_sprintf("%s/%s",filter->vacation_directory,oncelog->d_name);
+        if (Ustat(s,&properties)==0 && (properties.st_mtime+VACATION_MAX_DAYS*86400)<now)
+          Uunlink(s);
+        }
+      }
+    closedir(oncelogdir);
+    }
+  }
+
 while (parse_identifier(filter,CUS "require"))
   {
   /*
@@ -2160,13 +2736,27 @@ while (parse_identifier(filter,CUS "require"))
     }
   for (check=cap; check->character; ++check)
     {
-    if (eq_asciicase(check,&str_envelope,0)) filter->require_envelope=1;
-    else if (eq_asciicase(check,&str_fileinto,0)) filter->require_fileinto=1;
+    if (eq_octet(check,&str_envelope,0)) filter->require_envelope=1;
+    else if (eq_octet(check,&str_fileinto,0)) filter->require_fileinto=1;
 #ifdef SUBADDRESS
-    else if (eq_asciicase(check,&str_subaddress,0)) filter->require_subaddress=1;
+    else if (eq_octet(check,&str_subaddress,0)) filter->require_subaddress=1;
 #endif
-    else if (eq_asciicase(check,&str_comparator_ioctet,0)) ;
-    else if (eq_asciicase(check,&str_comparator_iascii_casemap,0)) ;
+#ifdef VACATION
+    else if (eq_octet(check,&str_vacation,0))
+      {
+      if (filter_test == FTEST_NONE && filter->vacation_directory == NULL)
+        {
+        filter->errmsg=CUS "vacation disabled";
+        return -1;
+        }
+      filter->require_vacation=1;
+      }
+#endif
+    else if (eq_octet(check,&str_copy,0)) filter->require_copy=1;
+    else if (eq_octet(check,&str_comparator_ioctet,0)) ;
+    else if (eq_octet(check,&str_comparator_iascii_casemap,0)) ;
+    else if (eq_octet(check,&str_comparator_enascii_casemap,0)) ;
+    else if (eq_octet(check,&str_comparator_iascii_numeric,0)) filter->require_iascii_numeric=1;
     else
       {
       filter->errmsg=CUS "unknown capability";
@@ -2194,6 +2784,9 @@ Arguments:
   filter      points to the entire file, read into store as a single string
   options     controls whether various special things are allowed, and requests
               special actions (not currently used)
+  sieve_vacation_directory  where to store vacation "once" files
+  useraddress string expression for :user part of address
+  subaddress  string expression for :subaddress part of address
   generated   where to hang newly-generated addresses
   error       where to pass back an error text
 
@@ -2206,8 +2799,8 @@ Returns:      FF_DELIVERED     success, a significant action was taken
 */
 
 int
-sieve_interpret(uschar *filter, int options, address_item **generated,
-  uschar **error)
+sieve_interpret(uschar *filter, int options, uschar *vacation_directory,
+  uschar *useraddress, uschar *subaddress, address_item **generated, uschar **error)
 {
 struct Sieve sieve;
 int r;
@@ -2218,6 +2811,24 @@ error = error;
 
 DEBUG(D_route) debug_printf("Sieve: start of processing\n");
 sieve.filter=filter;
+
+if (vacation_directory == NULL)
+  sieve.vacation_directory = NULL;
+else
+  {
+  sieve.vacation_directory=expand_string(vacation_directory);
+  if (sieve.vacation_directory == NULL)
+    {
+    *error = string_sprintf("failed to expand \"%s\" "
+      "(sieve_vacation_directory): %s", vacation_directory,
+      expand_string_message);
+    return FF_ERROR;
+    }
+  }
+
+sieve.useraddress = useraddress == NULL ? CUS "$local_part_prefix$local_part$local_part_suffix" : useraddress;
+sieve.subaddress = subaddress;
+
 #ifdef COMPILE_SYNTAX_CHECKER
 if (parse_start(&sieve,0,generated)==1)
 #else
@@ -2226,7 +2837,7 @@ if (parse_start(&sieve,1,generated)==1)
   {
   if (sieve.keep)
     {
-    add_addr(generated,US"inbox",1);
+    add_addr(generated,US"inbox",1,0,0,0);
     msg = string_sprintf("Keep");
     r = FF_DELIVERED;
     }
@@ -2243,13 +2854,13 @@ else
   r = FF_ERROR;
   *error = msg;
 #else
-  add_addr(generated,US"inbox",1);
+  add_addr(generated,US"inbox",1,0,0,0);
   r = FF_DELIVERED;
 #endif
   }
 
 #ifndef COMPILE_SYNTAX_CHECKER
-if (filter_test != NULL) printf("%s\n", (const char*) msg);
+if (filter_test != FTEST_NONE) printf("%s\n", (const char*) msg);
   else debug_printf("%s\n", msg);
 #endif
 
