@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/spam.h,v 1.3 2005/04/27 10:00:18 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003 - 2015 */
+/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003-???? */
 /* License: GPL */
 
 /* spam defines */
@@ -17,21 +19,12 @@
 
 /* SHUT_WR seems to be undefined on Unixware ? */
 #ifndef SHUT_WR
-# define SHUT_WR 1
+#define SHUT_WR 1
 #endif
 
-/* default weight */
-#define SPAMD_WEIGHT 1
-
-typedef struct spamd_address_container
-{
-  uschar * hostspec;
-  int is_rspamd:1;
-  int is_failed:1;
-  unsigned int weight;
-  unsigned int timeout;
-  unsigned int retry;
-  unsigned int priority;
+typedef struct spamd_address_container {
+  uschar tcp_addr[24];
+  unsigned int tcp_port;
 } spamd_address_container;
 
 #endif
