@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/string.c,v 1.7 2005/06/20 10:04:55 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Miscellaneous string-handling functions. Some are not required for
@@ -20,13 +22,12 @@ utilities and tests, and are cut out by the COMPILE_UTILITY macro. */
 /* This used just to be a regular expression, but with IPv6 things are a bit
 more complicated. If the address contains a colon, it is assumed to be a v6
 address (assuming HAVE_IPV6 is set). If a mask is permitted and one is present,
-the '/' in the input gets changed to 0, and maskptr points to its offset.
+and maskptr is not NULL, its offset is placed there.
 
 Arguments:
   s         a string
   maskptr   NULL if no mask is permitted to follow
-            otherwise, points to an int where the '/' offset is put, or
-            zero if there is no mask given
+            otherwise, points to an int where the offset of '/' is placed
 
 Returns:    0 if the string is not a textual representation of an IP address
             4 if it is an IPv4 address
@@ -39,8 +40,8 @@ string_is_ip_address(uschar *s, int *maskptr)
 int i;
 int yield = 4;
 
-/* If an optional mask is permitted, check for it. If found, change the / to
-a zero, and pass back the offset where this was done. */
+/* If an optional mask is permitted, check for it. If found, pass back the
+offset. */
 
 if (maskptr != NULL)
   {
@@ -49,11 +50,7 @@ if (maskptr != NULL)
   if (s != ss && isdigit(*(--ss)))
     {
     while (ss > s && isdigit(ss[-1])) ss--;
-    if (ss > s && *(--ss) == '/')
-      {
-      *ss = 0;
-      *maskptr = ss - s;
-      }
+    if (ss > s && *(--ss) == '/') *maskptr = ss - s;
     }
   }
 
@@ -79,16 +76,11 @@ if (Ustrchr(s, ':') != NULL)
   for (count = 0; count < 8; count++)
     {
     /* If the end of the string is reached before reading 8 components, the
-    address is valid provided a double colon has been read. Otherwise is
-    is invalid. */
-
-    if (*s == 0) return had_double_colon? yield : 0;
-
-    /* If the left of the current position is a valid IPv6 address and we
-    encounter a percent sign, the right of the current position is the
+    address is valid provided a double colon has been read. This also applies
+    if we hit the / that introduces a mask or the % that introduces the
     interface specifier (scope id) of a link-local address. */
 
-    if (*s == '%') return had_double_colon? yield : 0;
+    if (*s == 0 || *s == '%' || *s == '/') return had_double_colon? yield : 0;
 
     /* If a component starts with an additional colon, we have hit a double
     colon. This is permitted to appear once only, and counts as at least
@@ -131,10 +123,11 @@ if (Ustrchr(s, ':') != NULL)
     }
 
   /* If about to handle a trailing IPv4 address, drop through. Otherwise
-  all is well if we are at the end of the string or at a percent sign, which
-  introduces the interface specifier (scope id) of a link local address. */
+  all is well if we are at the end of the string or at the mask or at a percent
+  sign, which introduces the interface specifier (scope id) of a link local
+  address. */
 
-  if (!v4end) return (*s == 0 || *s == '%')? yield : 0;
+  if (!v4end) return (*s == 0 || *s == '%' || *s == '/')? yield : 0;
   }
 
 /* Test for IPv4 address, which may be the tail-end of an IPv6 address. */
@@ -146,7 +139,7 @@ for (i = 0; i < 4; i++)
   if (isdigit(*s) && isdigit(*(++s))) s++;
   }
 
-return (*s == 0)? yield : 0;
+return (*s == 0 || *s == '/')? yield : 0;
 }
 #endif  /* COMPILE_UTILITY */
 
@@ -275,14 +268,18 @@ return ch;
 
 /* This function is called for critical strings. It checks for any
 non-printing characters, and if any are found, it makes a new copy
-of the string with suitable escape sequences.
+of the string with suitable escape sequences. It is most often called by the
+macro string_printing(), which sets allow_tab TRUE.
 
-Argument:    input string
-Returns:     string with non-printers encoded as printing sequences
+Arguments:
+  s             the input string
+  allow_tab     TRUE to allow tab as a printing character
+
+Returns:        string with non-printers encoded as printing sequences
 */
 
 uschar *
-string_printing(uschar *s)
+string_printing2(uschar *s, BOOL allow_tab)
 {
 int nonprintcount = 0;
 int length = 0;
@@ -292,7 +289,7 @@ uschar *ss, *tt;
 while (*t != 0)
   {
   int c = *t++;
-  if (!mac_isprint(c)) nonprintcount++;
+  if (!mac_isprint(c) || (!allow_tab && c == '\t')) nonprintcount++;
   length++;
   }
 
@@ -311,7 +308,7 @@ tt = ss;
 while (*t != 0)
   {
   int c = *t;
-  if (mac_isprint(c)) *tt++ = *t++; else
+  if (mac_isprint(c) && (allow_tab || c != '\t')) *tt++ = *t++; else
     {
     *tt++ = '\\';
     switch (*t)
@@ -321,6 +318,7 @@ while (*t != 0)
       case '\b': *tt++ = 'b'; break;
       case '\v': *tt++ = 'v'; break;
       case '\f': *tt++ = 'f'; break;
+      case '\t': *tt++ = 't'; break;
       default: sprintf(CS tt, "%03o", *t); tt += 3; break;
       }
     t++;
@@ -338,7 +336,8 @@ return ss;
 *            Copy and save string                *
 *************************************************/
 
-/*
+/* This function assumes that memcpy() is faster than strcpy().
+
 Argument: string to copy
 Returns:  copy of string in new store
 */
@@ -346,8 +345,9 @@ Returns:  copy of string in new store
 uschar *
 string_copy(uschar *s)
 {
-uschar *ss = store_get(Ustrlen(s) + 1);
-Ustrcpy(ss, s);
+int len = Ustrlen(s) + 1;
+uschar *ss = store_get(len);
+memcpy(ss, s, len);
 return ss;
 }
 
@@ -357,7 +357,8 @@ return ss;
 *     Copy and save string in malloc'd store     *
 *************************************************/
 
-/*
+/* This function assumes that memcpy() is faster than strcpy().
+
 Argument: string to copy
 Returns:  copy of string in new store
 */
@@ -365,8 +366,9 @@ Returns:  copy of string in new store
 uschar *
 string_copy_malloc(uschar *s)
 {
-uschar *ss = store_malloc(Ustrlen(s) + 1);
-Ustrcpy(ss, s);
+int len = Ustrlen(s) + 1;
+uschar *ss = store_malloc(len);
+memcpy(ss, s, len);
 return ss;
 }
 
@@ -867,7 +869,7 @@ if (p + count >= *size)
 
   else if (!store_extend(string, oldsize, *size))
     {
-    BOOL release_ok = store_last_get == string;
+    BOOL release_ok = store_last_get[store_pool] == string;
     uschar *newstring = store_get(*size);
     memcpy(newstring, string, p);
     if (release_ok) store_release(string);
@@ -887,6 +889,49 @@ return string;
 
 
 
+#ifndef COMPILE_UTILITY
+/*************************************************
+*        Append strings to another string        *
+*************************************************/
+
+/* This function can be used to build a string from many other strings.
+It calls string_cat() to do the dirty work.
+
+Arguments:
+  string   points to the start of the string that is being built, or NULL
+             if this is a new string that has no contents yet
+  size     points to a variable that holds the current capacity of the memory
+             block (updated if changed)
+  ptr      points to a variable that holds the offset at which to add
+             characters, updated to the new offset
+  count    the number of strings to append
+  ...      "count" uschar* arguments, which must be valid zero-terminated
+             C strings
+
+Returns:   pointer to the start of the string, changed if copied for expansion.
+           The string is not zero-terminated - see string_cat() above.
+*/
+
+uschar *
+string_append(uschar *string, int *size, int *ptr, int count, ...)
+{
+va_list ap;
+int i;
+
+va_start(ap, count);
+for (i = 0; i < count; i++)
+  {
+  uschar *t = va_arg(ap, uschar *);
+  string = string_cat(string, size, ptr, t, Ustrlen(t));
+  }
+va_end(ap);
+
+return string;
+}
+#endif
+
+
+
 /*************************************************
 *        Format a string with length checks      *
 *************************************************/
@@ -899,9 +944,9 @@ on whether the variable length list of data arguments are given explicitly or
 as a va_list item.
 
 The formats are the usual printf() ones, with some omissions (never used) and
-two additions for strings: %S forces lower case, %#s or %#S prints nothing for
-a NULL string. Without the # "NULL" is printed (useful in debugging). There is
-also the addition of %D, which inserts the date in the form used for
+two additions for strings: %S forces lower case, and %#s or %#S prints nothing
+for a NULL string. Without the # "NULL" is printed (useful in debugging). There
+is also the addition of %D, which inserts the date in the form used for
 datestamped log files.
 
 Arguments:
@@ -928,6 +973,8 @@ return yield;
 BOOL
 string_vformat(uschar *buffer, int buflen, char *format, va_list ap)
 {
+enum { L_NORMAL, L_SHORT, L_LONG, L_LONGLONG, L_LONGDOUBLE };
+
 BOOL yield = TRUE;
 int width, precision;
 char *fp = format;             /* Deliberately not unsigned */
@@ -940,6 +987,7 @@ string_datestamp_offset = -1;  /* Datestamp not inserted */
 
 while (*fp != 0)
   {
+  int length = L_NORMAL;
   int *nptr;
   int slen;
   char *null = "NULL";         /* ) These variables */
@@ -993,7 +1041,25 @@ while (*fp != 0)
       }
     }
 
-  if (strchr("hlL", *fp) != NULL) fp++;
+  /* Skip over 'h', 'L', 'l', and 'll', remembering the item length */
+
+  if (*fp == 'h')
+    { fp++; length = L_SHORT; }
+  else if (*fp == 'L')
+    { fp++; length = L_LONGDOUBLE; }
+  else if (*fp == 'l')
+    {
+    if (fp[1] == 'l')
+      {
+      fp += 2;
+      length = L_LONGLONG;
+      }
+    else
+      {
+      fp++;
+      length = L_LONG;
+      }
+    }
 
   /* Handle each specific format type. */
 
@@ -1009,10 +1075,21 @@ while (*fp != 0)
     case 'u':
     case 'x':
     case 'X':
-    if (p >= last - 12) { yield = FALSE; goto END_FORMAT; }
+    if (p >= last - ((length > L_LONG)? 24 : 12))
+      { yield = FALSE; goto END_FORMAT; }
     strncpy(newformat, item_start, fp - item_start);
     newformat[fp - item_start] = 0;
-    sprintf(CS p, newformat, va_arg(ap, int));
+
+    /* Short int is promoted to int when passing through ..., so we must use
+    int for va_arg(). */
+
+    switch(length)
+      {
+      case L_SHORT:
+      case L_NORMAL:   sprintf(CS p, newformat, va_arg(ap, int)); break;
+      case L_LONG:     sprintf(CS p, newformat, va_arg(ap, long int)); break;
+      case L_LONGLONG: sprintf(CS p, newformat, va_arg(ap, LONGLONG_T)); break;
+      }
     while (*p) p++;
     break;
 
@@ -1025,9 +1102,11 @@ while (*fp != 0)
     break;
 
     /* %f format is inherently insecure if the numbers that it may be
-    handed are unknown (e.g. 1e300). However, in Exim, the only use of %f
-    is for printing load averages, and these are actually stored as integers
-    (load average * 1000) so the size of the numbers is constrained. */
+    handed are unknown (e.g. 1e300). However, in Exim, %f is used for
+    printing load averages, and these are actually stored as integers
+    (load average * 1000) so the size of the numbers is constrained.
+    It is also used for formatting sending rates, where the simplicity
+    of the format prevents overflow. */
 
     case 'f':
     case 'e':
@@ -1038,7 +1117,10 @@ while (*fp != 0)
     if (p >= last - precision - 8) { yield = FALSE; goto END_FORMAT; }
     strncpy(newformat, item_start, fp - item_start);
     newformat[fp-item_start] = 0;
-    sprintf(CS p, newformat, va_arg(ap, double));
+    if (length == L_LONGDOUBLE)
+      sprintf(CS p, newformat, va_arg(ap, long double));
+    else
+      sprintf(CS p, newformat, va_arg(ap, double));
     while (*p) p++;
     break;
 
@@ -1126,7 +1208,6 @@ return yield;
 
 
 
-
 #ifndef COMPILE_UTILITY
 /*************************************************
 *       Generate an "open failed" message        *
