@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/transports/smtp.h,v 1.5 2005/03/08 15:32:02 tom Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options and other private data. */
@@ -30,6 +32,7 @@ typedef struct {
   int     final_timeout;
   int     size_addition;
   int     hosts_max_try;
+  int     hosts_max_try_hardlimit;
   BOOL    allow_localhost;
   BOOL    gethostbyname;
   BOOL    dns_qualify_single;
@@ -41,11 +44,20 @@ typedef struct {
   BOOL    retry_include_ip_address;
   #ifdef SUPPORT_TLS
   uschar *tls_certificate;
+  uschar *tls_crl;
   uschar *tls_privatekey;
   uschar *tls_require_ciphers;
   uschar *tls_verify_certificates;
   BOOL    tls_tempfail_tryclear;
   #endif
+  #ifdef EXPERIMENTAL_DOMAINKEYS
+  uschar *dk_domain;
+  uschar *dk_private_key;
+  uschar *dk_selector;
+  uschar *dk_canon;
+  uschar *dk_headers;
+  uschar *dk_strict;
+  #endif
 } smtp_transport_options_block;
 
 /* Data for reading the private options. */
